@@ -3,9 +3,11 @@ use blockfrost::{load, BlockFrostApi, BlockFrostSettings, QueryOrder};
 fn build_api() -> blockfrost::Result<BlockFrostApi> {
     let configurations = load::configurations_from_env()?;
     let project_id = configurations["project_id"].as_str().unwrap();
-    let settings = BlockFrostSettings::new().configure(|query| {
-        query.set_count(5).set_page(10).set_order(QueryOrder::Descending);
-    });
+    let settings = {
+        let mut settings = BlockFrostSettings::new();
+        settings.query_parameters.set_count(5).set_page(10).set_order(QueryOrder::Descending);
+        settings
+    };
     let api = BlockFrostApi::new(project_id, settings);
     Ok(api)
 }
