@@ -7,12 +7,12 @@ use blockfrost::{load, stream::StreamExt, BlockFrostApi, BlockFrostSettings};
 fn build_api() -> blockfrost::Result<BlockFrostApi> {
     let configurations = load::configurations_from_env()?;
     let project_id = configurations["project_id"].as_str().unwrap();
-    let settings = BlockFrostSettings::new().configure(|query| {
-        // Show 3 elements per page (just for this example)
-        query.set_count(3);
-    });
-    let api = BlockFrostApi::new(project_id, settings);
 
+    let mut settings = BlockFrostSettings::new();
+    // Show 3 elements per page (just for this example)
+    settings.query_parameters.set_count(3);
+
+    let api = BlockFrostApi::new(project_id, settings);
     Ok(api)
 }
 
