@@ -1,7 +1,8 @@
 use reqwest::{header::HeaderValue, Body, Method};
 use serde::{Deserialize, Serialize};
+use serde_json::from_str as json_from;
 
-use crate::{url::Url, *};
+use crate::{request::send_request, url::Url, *};
 
 impl BlockFrostApi {
     /// Obtain information about Move Instantaneous Rewards (MIRs) of a specific transaction.
@@ -16,21 +17,20 @@ impl BlockFrostApi {
         let endpoint_suffix = "/tx/submit";
         let Url(url) = Url::from_endpoint_without_parameters(&self.settings, endpoint_suffix);
 
-        let response = self
+        let request = self
             .client
-            .request(Method::POST, url)
+            .request(Method::POST, &url)
             .header(content_type_header.0, content_type_header.1)
-            .body(body)
-            .send()
-            .await?;
+            .body(body);
 
-        let status_code = response.status();
-        let text = response.text().await?;
+        let (status, text) = send_request(request, self.settings.retry_settings)
+            .await
+            .map_err(|reason| Error::Reqwest { url: url.clone(), reason })?;
 
-        if !status_code.is_success() {
-            return Err(process_error_response(&text, status_code));
+        if !status.is_success() {
+            return Err(process_error_response(&text, status, &url));
         }
-        Ok(serde_json::from_str(&text)?)
+        Ok(json_from(&text).map_err(|reason| json_error(url, text, reason))?)
     }
 
     endpoints! {
