@@ -1,13 +1,16 @@
-use reqwest::{
-    multipart::{Form, Part},
-    Client,
-};
+use reqwest::multipart::{Form, Part};
 use serde::{Deserialize, Serialize};
+use serde_json::from_str as json_from;
 
-use crate::{error::process_error_response, utils::build_header_map, Integer, IpfsSettings};
+use crate::{
+    error::{json_error, process_error_response, reqwest_error},
+    request::{send_request, send_request_unprocessed},
+    utils::create_client_with_project_id,
+    Integer, IpfsSettings, RetrySettings,
+};
 
 /// Provides methods for making requests to the
-/// [`IPFS API`](https://docs.blockfrost.io/#tag/IPFS-Add).
+/// [IPFS API](https://docs.blockfrost.io/#tag/IPFS-Add).
 #[derive(Debug, Clone)]
 pub struct IpfsApi {
     client: reqwest::Client,
@@ -25,8 +28,7 @@ impl IpfsApi {
     /// [`HeaderValue`]: reqwest::header::HeaderValue
     /// [`HeaderValue::from_str`]: reqwest::header::HeaderValue::from_str
     pub fn new(project_id: impl AsRef<str>, settings: IpfsSettings) -> Self {
-        let header_map = build_header_map(project_id.as_ref());
-        let client = Client::builder().default_headers(header_map).build().unwrap();
+        let client = create_client_with_project_id(project_id.as_ref());
         Self { client, settings }
     }
 
@@ -46,16 +48,16 @@ impl IpfsApi {
         let form = Form::new().part("file", part);
 
         let request = self.client.post(&url).multipart(form);
-        let response = request.send().await?;
 
-        let status_code = response.status();
-        let text = response.text().await?;
+        let (status, text) = send_request(request, self.settings.retry_settings)
+            .await
+            .map_err(|reason| reqwest_error(&url, reason))?;
 
-        if !status_code.is_success() {
-            return Err(process_error_response(&text, status_code));
+        if !status.is_success() {
+            return Err(process_error_response(&text, status, &url));
         }
 
-        Ok(serde_json::from_str(&text)?)
+        Ok(json_from(&text).map_err(|reason| json_error(url, text, reason))?)
     }
 
     /// Retrieve an object from the IFPS gateway.
@@ -69,18 +71,20 @@ impl IpfsApi {
         let url = self.settings.network_address.clone()
             + &format!("/ipfs/gateway/{IPFS_path}", IPFS_path = ipfs_path);
 
-        let response = self.client.get(&url).send().await?;
-        let status_code = response.status();
-        let bytes = response.bytes().await?;
+        let request = self.client.get(&url);
 
-        if !status_code.is_success() {
-            // Safety:
-            //   The error responses are guaranteed to be UTF-8.
-            let text = std::str::from_utf8(&bytes).unwrap();
-            return Err(process_error_response(text, status_code));
+        let response = send_request_unprocessed(request, self.retry_settings())
+            .await
+            .map_err(|reason| reqwest_error(&url, reason))?;
+        let status = response.status();
+
+        if !status.is_success() {
+            let text = response.text().await.map_err(|reason| reqwest_error(&url, reason))?;
+            Err(process_error_response(&text, status, &url))
+        } else {
+            let bytes = response.bytes().await.map_err(|reason| reqwest_error(&url, reason))?;
+            Ok(bytes.to_vec())
         }
-
-        Ok(bytes.to_vec())
     }
 
     /// Pinned objects are counted in your user storage quota.
@@ -92,14 +96,15 @@ impl IpfsApi {
         let url = self.settings.network_address.clone()
             + &format!("/ipfs/pin/add/{IPFS_path}", IPFS_path = ipfs_path);
 
-        let response = self.client.post(&url).send().await?;
-        let status_code = response.status();
-        let text = response.text().await?;
-        if !status_code.is_success() {
-            return Err(process_error_response(&text, status_code));
+        let request = self.client.post(&url);
+        let (status, text) = send_request(request, self.settings.retry_settings)
+            .await
+            .map_err(|reason| reqwest_error(&url, reason))?;
+        if !status.is_success() {
+            return Err(process_error_response(&text, status, &url));
         }
 
-        Ok(serde_json::from_str(&text)?)
+        Ok(json_from(&text).map_err(|reason| json_error(url, text, reason))?)
     }
 
     /// List objects pinned to local storage.
@@ -110,14 +115,16 @@ impl IpfsApi {
     pub async fn pin_list(&self) -> crate::Result<Vec<IpfsPinList>> {
         let url = self.settings.network_address.clone() + "/ipfs/pin/list";
 
-        let response = self.client.get(&url).send().await?;
-        let status_code = response.status();
-        let text = response.text().await?;
-        if !status_code.is_success() {
-            return Err(process_error_response(&text, status_code));
+        let request = self.client.get(&url);
+        let (status, text) = send_request(request, self.settings.retry_settings)
+            .await
+            .map_err(|reason| reqwest_error(&url, reason))?;
+
+        if !status.is_success() {
+            return Err(process_error_response(&text, status, &url));
         }
 
-        Ok(serde_json::from_str(&text)?)
+        Ok(json_from(&text).map_err(|reason| json_error(url, text, reason))?)
     }
 
     /// Get information about locally pinned IPFS object.
@@ -129,14 +136,16 @@ impl IpfsApi {
         let url = self.settings.network_address.clone()
             + &format!("/ipfs/pin/list/{IPFS_path}", IPFS_path = ipfs_path);
 
-        let response = self.client.get(&url).send().await?;
-        let status_code = response.status();
-        let text = response.text().await?;
-        if !status_code.is_success() {
-            return Err(process_error_response(&text, status_code));
+        let request = self.client.get(&url);
+        let (status, text) = send_request(request, self.settings.retry_settings)
+            .await
+            .map_err(|reason| reqwest_error(&url, reason))?;
+
+        if !status.is_success() {
+            return Err(process_error_response(&text, status, &url));
         }
 
-        Ok(serde_json::from_str(&text)?)
+        Ok(json_from(&text).map_err(|reason| json_error(url, text, reason))?)
     }
 
     /// Remove pinned objects from local storage.
@@ -148,14 +157,20 @@ impl IpfsApi {
         let url = self.settings.network_address.clone()
             + &format!("/ipfs/pin/remove/{IPFS_path}", IPFS_path = ipfs_path);
 
-        let response = self.client.post(&url).send().await?;
-        let status_code = response.status();
-        let text = response.text().await?;
-        if !status_code.is_success() {
-            return Err(process_error_response(&text, status_code));
+        let request = self.client.post(&url);
+        let (status, text) = send_request(request, self.settings.retry_settings)
+            .await
+            .map_err(|reason| reqwest_error(&url, reason))?;
+
+        if !status.is_success() {
+            return Err(process_error_response(&text, status, &url));
         }
 
-        Ok(serde_json::from_str(&text)?)
+        Ok(json_from(&text).map_err(|reason| json_error(url, text, reason))?)
+    }
+
+    pub(crate) fn retry_settings(&self) -> RetrySettings {
+        self.settings.retry_settings
     }
 }
 
